# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=E0201,W0131,E0213,F0203,W0232,C0103,R0904,W0613
# pylint-version = 0.7.0
#
# Copyright 2004-2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.cli

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base
_tests = []


#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for cli tests """
    LOAD_MODULES = (('_cli', 'svnmailer.cli'),)

    def getInitialMocks(self):
        """ Returns the initially mocked modules """
        return [
            self._mock_svnmailer_settings(),
            self._mock_svnmailer_util(),
            self._mock_svnmailer_subversion(),
            self._mock_svnmailer_processes(),
            self._mock_textwrap(),
            self._mock_optparse(),
            self._mock_os(),
            self._mock_signal(),
            self._mock_warnings(),
        ]


#### Test Cases                                     ####
########################################################
class ErrorTestCase(TestBase):
    """ svnmailer.cli.Error """

    def test00general(self):
        """ Error is a public Exception """
        expected = Exception
        result = self._cli.Error

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._cli),
            "Is not public")

_tests.append(ErrorTestCase)


class CommandlineErrorTestCase(TestBase):
    """ svnmailer.cli.CommandlineError """

    def test00general(self):
        """ CommandlineError is a public Error """
        expected = self._cli.Error
        result = self._cli.CommandlineError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._cli),
            "Is not public")

_tests.append(CommandlineErrorTestCase)


class OptionParserTestCase(TestBase):
    """ svnmailer.cli.OptionParser """
    PREFIX = "OptionParser"

    def setUp(self):
        """ extend test environment """
        super(OptionParserTestCase, self).setUp()

        class Helper(object):
            def __init__(this):
                this.args = list('abc')
                this.svnversion = self.mock['svnmailer.subversion'].version
            def fixUp(this, (options, fixed)):
                if self.mock.error == "cli.fixup":
                    raise self._cli.CommandlineError()
                self.mock.result.extend(['OptionHelper.fixUp', options, fixed])
                return options
            def formatOldStyle(this, width):
                self.mock.result.extend(['OptionHelper.formatOldStyle', width])
                return 'oldstyle stuff'

        class MyParser0(self._cli.OptionParser):
            def _createSvnmailerOptionHelper(this, background):
                if self.mock.error == 'cli':
                    raise self._cli.CommandlineError()
                self.mock.result.extend(['_createSvnmailerOptionHelper',
                    background])
                return "helper"

        class MyParser(self._cli.OptionParser):
            def __init__(this):
                this._svnmailer_helper = Helper()

        self._myparser0 = MyParser0
        self._myparser = MyParser

    def test00general(self):
        """ OptionParser is a public optparse.OptionParser """
        expected = self.mock['optparse'].OptionParser
        result = self._cli.OptionParser

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._cli),
            "Is not public")

    def test01init(self):
        """ .__init__ initializes parser and helper """
        inst = self._myparser0()
        self.mock.result.append(inst._svnmailer_helper)
        inst = self._myparser0(False)
        self.mock.result.append(inst._svnmailer_helper)

        self.expect(
            'optparse.OptionParser', (),
                {'usage': '1', 'prog': '<prog>', 'version': '2'},
            '_createSvnmailerOptionHelper', True,
            'helper',
            'optparse.OptionParser', (),
                {'usage': '1', 'prog': '<prog>', 'version': '2'},
            '_createSvnmailerOptionHelper', False,
            'helper'
        )

    def test02init(self):
        """ .__init__ passes CommandlineError """
        self.mock.error = 'cli'
        self.failUnlessRaises(self._cli.CommandlineError, self._myparser0, [])

    def test03parseArgs(self):
        """ .parseArgs calls parse_args and fixes the result up """
        inst = self._myparser()
        self.mock.result.append(inst.parseArgs())

        self.expect(
            'optparse.OptionParser.parse_args', ['a', 'b', 'c'], (), {},
            'OptionHelper.fixUp', 'options', [],
            'options'
        )

    def test04parseArgs(self):
        """ .parseArgs passes CommandlineError """
        self.mock.error = "cli.fixup"
        inst = self._myparser()
        self.failUnlessRaises(self._cli.CommandlineError, inst.parseArgs)

    def test05error(self):
        """ .error raises CommandlineError """
        self.failUnlessRaises(self._cli.CommandlineError,
            self._myparser().error, 'foo')

    def test06getversion(self):
        """ .get_version returns the proper version string """
        import svnmailer
        old_version = svnmailer.version
        try:
            svnmailer.version = base.Store(string = "3.14159265")
            self.mock.result.append(self._myparser().get_version())
        finally:
            svnmailer.version = old_version
 
        self.expect('svnmailer-3.14159265\nwith svn 1.2.3 (456789)')

    def test07getusage(self):
        """ .get_usage returns the proper usage string """
        class MyParser(self._myparser):
            def get_prog_name(this):
                return "gorp"
        self.mock.result.append(MyParser().get_usage())

        self.expect('Usage: gorp <options>\n')

    def test08getprogname(self):
        """ .get_prog_name returns the program name """
        inst = self._myparser()
        inst.prog = "fooo"
        self.mock.result.append(inst.get_prog_name())

        self.expect('fooo')

    def test09getprogname(self):
        """ ._get_prog_name returns the program name """
        inst = self._myparser()
        inst.prog = "baaar"
        self.mock.result.append(inst._get_prog_name())

        self.expect('baaar')

    def test10formathelp(self):
        """ .format_help returns the help string """
        class MyParser(self._myparser):
            def _createHelpFormatter(this, *args, **kwargs):
                self.mock.result.extend(['_createHelpFormatter', args, kwargs])
                return "formatter"
        inst = MyParser()
        self.mock.result.append(inst.format_help())

        self.expect(
            '_createHelpFormatter', (), {'width': 68},
            'optparse.OptionParser.format_help', 'formatter',
            'OptionHelper.formatOldStyle', 68,
            'formatted help\noldstyle stuff'
        )

    def test11formathelp(self):
        """ .format_help deals with small terminals """
        self.mock.optparse.minwidth = 85
        self.mock.mailer_util.terminal_width = 60
        class MyParser(self._myparser):
            def _createHelpFormatter(this, width):
                self.mock.result.extend(['_createHelpFormatter', width])
                self.mock.optparse.width = width
                return "formatter [%s]" % width
        inst = MyParser()
        inst.format_help()

        self.expect(
            '_createHelpFormatter', 59,
            '_createHelpFormatter', 79,
            '_createHelpFormatter', 89,
            'optparse.OptionParser.format_help', 'formatter [89]',
            'OptionHelper.formatOldStyle', 89
        )

    def test12createSvnmailerOptionHelper(self):
        """ ._createSvnmailerOptionHelper returns an OptionHelper """
        class OptionHelper(object):
            def __init__(this, parser, background):
                this._mocked = "OptionHelperMocked"
                self.mock.result.extend(['OptionHelper',
                    parser, background])
        self._cli.OptionHelper = OptionHelper
        inst = self._myparser()
        self.mock.result.append(
            inst._createSvnmailerOptionHelper('*background')._mocked
        )

        self.expect(
            'OptionHelper', inst, '*background',
            "OptionHelperMocked"
        )

    def test13createHelpFormatter(self):
        """ ._createHelpFormatter returns an IndentedHelpFormatter """
        inst = self._myparser()
        self.mock.result.append(
            inst._createHelpFormatter('*args', foo = 'bar')._mocked
        )

        self.expect(
            'optparse.IndentedHelpFormatter', ('*args',), {'foo': 'bar'},
            'IndentedHelpFormatterMocked'
        )

_tests.append(OptionParserTestCase)


class OptionHelperTestCase(TestBase):
    """ svnmailer.cli.OptionHelper """
    PREFIX = "OptionHelper"

    def setUp(self):
        """ Extend test environment """
        super(OptionHelperTestCase, self).setUp()

        class OptionGroup:
            def add_option(this, *args, **kwargs):
                self.mock.result.extend(['OptionGroup.add_option',
                    args, kwargs])

        class Parser(object):
            def get_prog_name(this):
                return 'gorp'
            def add_option_group(this, *args, **kwargs):
                self.mock.result.extend(['Parser.add_option_group',
                    args, kwargs])
                return OptionGroup()

        class MyHelper(self._cli.OptionHelper):
            _COMMON_TITLE = "title:common"
            _BEHAVIOR_TITLE = "title:behavior"
            _SUPPLEMENTAL_TITLE = "title:supp"

            def __init__(this):
                this._parser = Parser()
                this._background = True

        self._myhelper = MyHelper

    def test00general(self):
        """ OptionHelper is a private object """
        expected = object
        result = self._cli.OptionHelper

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failIf(result.__name__ in self.public(self._cli),
            "Is public")

    def test01initial(self):
        """ OptionHelper initially provides proper class attributes """
        cls = self._cli.OptionHelper
        self.mock.result = [
            cls._WIN32_BG_ARG,
            cls._COMMON_TITLE, cls._BEHAVIOR_TITLE, cls._SUPPLEMENTAL_TITLE,
            cls._PATH_OPTIONS, cls._REQUIRED_OPTIONS,
            cls._OLD_OPTIONS, cls._OLD_OPTIONS_1_2,
        ]

        self.expect(
            'bg-process',
            'COMMON PARAMETERS', 'BEHAVIOR OPTIONS', 'SUPPLEMENTAL PARAMETERS',
            (('repository', '--repository'), ('config', '--config')),
            (
                ('repository', None, 'repository path'),
                ('revision', ('C', 'P'), 'revision number'),
                ('author', ('P', 'L', 'U'), 'author parameter'),
                ('propname', ('P',), 'property name parameter')
            ),
            {'commit': ('--commit', '--repository', '--revision', '--config'),
                'propchange': ('--propchange', '--repository', '--revision',
                '--author', '--propname', '--config')},
            {'propchange2': ('--propchange', '--repository', '--revision',
                '--author', '--propname', '--action', '--config'),
                'lock': ('--lock', '--repository', '--author', '--config'),
                'unlock': ('--unlock', '--repository', '--author', '--config')}
        )

    def test02init(self):
        """ .__init__ initializes attributes and options """
        import sys

        class MyHelper(self._cli.OptionHelper):
            def _transformArgs(this, args):
                self.mock.result.extend(['_transformArgs', args])
                return ["_%s_" % arg for arg in args]
            def _addOptions(this):
                self.mock.result.append('_addOptions')
        parser = base.Store()
        old_argv = sys.argv
        try:
            sys.argv = ['/foo/bar', 'a', 'b', 'c']
            inst = MyHelper(parser, '*background')
        finally:
            sys.argv = old_argv

        self.mock.result.extend([
            inst._parser, parser.prog, inst._background, inst.args
        ])

        self.expect(
            'os.path.basename', '/foo/bar',
            '_transformArgs', ['a', 'b', 'c'],
            '_addOptions',
            parser, 'bar', '*background', ['_a_', '_b_', '_c_']
        )

    def test03init(self):
        """ .__init__ passes CommandlineError """
        import sys

        class MyHelper(self._cli.OptionHelper):
            def _transformArgs(this, args):
                raise self._cli.CommandlineError()
        parser = base.Store()
        old_argv = sys.argv
        try:
            sys.argv = ['/foo/bar', 'a', 'b', 'c']
            self.failUnlessRaises(self._cli.CommandlineError,
                MyHelper, parser, '*b')
        finally:
            sys.argv = old_argv

    def test04fixUp(self):
        """ .fixUp adjusts the options """
        class MyHelper(self._myhelper):
            def _ensureRequired(this, options):
                self.mock.result.extend(['_ensureRquired', options])
            def _delocalize(this, options):
                self.mock.result.extend(['_delocalize', options])
            def _handleBackground(this, options):
                self.mock.result.extend(['_handleBackground', options])
                return options
        inst = MyHelper()
        self.mock.result.append(inst.fixUp(('*options', [])))

        self.expect(
            '_ensureRquired', '*options',
            '_delocalize', '*options',
            '_handleBackground', '*options',
            '*options'
        )

    def test05fixUp(self):
        """ .fixUp respects the fixed win32bg argument """
        class MyHelper(self._myhelper):
            def _ensureRequired(this, options):
                self.mock.result.extend(['_ensureRquired', this._background])
            def _delocalize(this, options):
                self.mock.result.extend(['_delocalize', this._background])
            def _handleBackground(this, options):
                self.mock.result.extend(['_handleBackground', this._background])
                return options
        inst = MyHelper()
        inst._background = True
        options = base.Store(background = True)
        self.mock.result.append(inst.fixUp((options, ['bg-process'])))

        self.expect(
            '_ensureRquired', False,
            '_delocalize', False,
            '_handleBackground', False,
            options
        )

    def test06fixUp(self):
        """ .fixUp rejects fixed arguments """
        options = base.Store(background = True)
        self.failUnlessRaises(self._cli.CommandlineError,
            self._myhelper().fixUp, (options, ['a']))

    def test07fixUp(self):
        """ .fixUp sets the action attribute """
        class MyHelper(self._myhelper):
            def _ensureRequired(this, options):
                pass
            def _delocalize(this, options):
                pass
            def _handleBackground(this, options):
                return options
        inst = MyHelper()
        options = base.Store()
        self.mock['svnmailer.subversion'].version.min_1_2 = False
        self.mock.result.append(inst.fixUp((options, [])).action)

        self.expect(None)

    def test08formatOldStyle(self):
        """ .formatOldStyle returns the formatted 1.2 oldstyle description """
        inst = self._myhelper()
        self.mock.result.append(inst.formatOldStyle(82))

        self.expect(
            'textwrap.TextWrapper', {'width': 82},
            'textwrap.TextWrapper.fill', {'width': 82},
                "Alternatively you can use the old style compatibility "
                "command lines (options described above don't apply then):",
            'textwrap.TextWrapper.fill',
                {'width': 82, 'subsequent_indent': '     '}, '',
            'textwrap.TextWrapper.fill',
                {'width': 82, 'subsequent_indent': '     '},
                'gorp commit <repos> <revision> [<config>]',
            'textwrap.TextWrapper.fill',
                {'width': 82, 'subsequent_indent': '     '},
                'gorp propchange <repos> <revision> <author> <propname> '
                '[<config>]',
            'textwrap.TextWrapper.fill',
                {'width': 82, 'subsequent_indent': '     '}, '',
            'textwrap.TextWrapper.fill',
                {'width': 82, 'subsequent_indent': '     '},
                    'svn 1.2 and later:',
            'textwrap.TextWrapper.fill',
                {'width': 82, 'subsequent_indent': '     '},
                'gorp propchange2 <repos> <revision> <author> <propname> '
                '<action> [<config>]',
            'textwrap.TextWrapper.fill',
                {'width': 82, 'subsequent_indent': '     '},
                'gorp lock <repos> <author> [<config>]',
            'textwrap.TextWrapper.fill',
                {'width': 82, 'subsequent_indent': '     '},
                'gorp unlock <repos> <author> [<config>]',
            "BROKEN(Alternatively you can use the old style compatibility "
                "command lines (options described above don't apply then):)\n"
                "BROKEN()\n"
                "BROKEN(gorp commit <repos> <revision> [<config>])\n"
                "BROKEN(gorp propchange <repos> <revision> <author> "
                "<propname> [<config>])\n"
                "BROKEN()\n"
                "BROKEN(svn 1.2 and later:)\n"
                "BROKEN(gorp propchange2 <repos> <revision> <author> "
                "<propname> <action> [<config>])\n"
                "BROKEN(gorp lock <repos> <author> [<config>])\n"
                "BROKEN(gorp unlock <repos> <author> [<config>])\n"
            )

    def test09formatOldStyle(self):
        """ .formatOldStyle returns the formatted 1.1 oldstyle description """
        inst = self._myhelper()
        self.mock['svnmailer.subversion'].version.min_1_2 = False
        self.mock.result.append(inst.formatOldStyle(83))

        self.expect(
            'textwrap.TextWrapper', {'width': 83},
            'textwrap.TextWrapper.fill', {'width': 83},
                "Alternatively you can use the old style compatibility "
                "command lines (options described above don't apply then):",
            'textwrap.TextWrapper.fill',
                {'width': 83, 'subsequent_indent': '     '}, '',
            'textwrap.TextWrapper.fill',
                {'width': 83, 'subsequent_indent': '     '},
                'gorp commit <repos> <revision> [<config>]',
            'textwrap.TextWrapper.fill',
                {'width': 83, 'subsequent_indent': '     '},
                'gorp propchange <repos> <revision> <author> <propname> '
                '[<config>]',
            "BROKEN(Alternatively you can use the old style compatibility "
                "command lines (options described above don't apply then):)\n"
                "BROKEN()\n"
                "BROKEN(gorp commit <repos> <revision> [<config>])\n"
                "BROKEN(gorp propchange <repos> <revision> <author> "
                "<propname> [<config>])\n"
        )

    def test10handleBackground(self):
        """ ._handleBackground passes through options """
        inst = self._myhelper()
        inst._background = True
        opts = base.Store(background = False, _mocked = self)
        self.mock.result.append(inst._handleBackground(opts)._mocked)
        inst._background = False
        opts.background = True
        self.mock.result.append(inst._handleBackground(opts)._mocked)

        self.expect(self, self)

    def test11handleBackground(self):
        """ ._handleBackground forks once and the parent exits """
        opts = base.Store(background = True)
        self._myhelper()._handleBackground(opts)

        self.expect('signal', 1, 1, 'os.fork', 'os._exit', 0)

    def test12handleBackground(self):
        """ ._handleBackground forks twice and the first child exits """
        self.mock.os.forkpids = [0, 31515]
        opts = base.Store(background = True)
        self._myhelper()._handleBackground(opts)

        self.expect('signal', 1, 1, 'os.fork', 'os.fork', 'os._exit', 0)

    def test13handleBackground(self):
        """ ._handleBackground forks twice and the second child continues """
        self.mock.os.forkpids = [0, 0]
        opts = base.Store(background = True)
        self.mock.result.append(
            self._myhelper()._handleBackground(opts).background
        )

        self.expect('signal', 1, 1, 'os.fork', 'os.fork', 'os.setsid', True)

    def test14handleBackground(self):
        """ ._handleBackground catches OSError """
        opts = base.Store(background = True)
        self.mock.error = 'os.fork'
        self._myhelper()._handleBackground(opts)

        self.expect(
            'signal', 1, 1,
            'warnings.warn',
                'svnmailer: OSError while detaching from foreground: '
                '[Errno 12] out of memory...', None, None
        )

    def test15handleBackground(self):
        """ ._handleBackground catches AttributeError """
        opts = base.Store(background = True)
        self.addMock('signal', {})
        import sys
        old_platform = sys.platform
        try:
            sys.platform = 'foo'
            self._myhelper()._handleBackground(opts)
        finally:
            sys.platform = old_platform

        self.expect(
            'warnings.warn',
                'svnmailer: --background is not implemented on this platform',
                None, None
        )

    def test16handleBackground(self):
        """ ._handleBackground catches ImportError """
        opts = base.Store(background = True)
        self.addMock('signal', None)
        import sys
        old_platform = sys.platform
        try:
            sys.platform = 'foo'
            self._myhelper()._handleBackground(opts)
        finally:
            sys.platform = old_platform

        self.expect(
            'warnings.warn',
                'svnmailer: --background is not implemented on this platform',
                None, None
        )

    def test17handleBackground(self):
        """ ._handleBackground respawns the process on Win32 """
        opts = base.Store(background = True)
        self.addMock('signal', None)
        import sys
        old_platform = sys.platform
        old_argv = sys.argv
        old_exe = sys.executable
        try:
            sys.platform = "win32"
            sys.argv = ['python.exe', 'a', 'b', '--background', 'c', '-b']
            sys.executable = 'foo/python2.3'
            self._myhelper()._handleBackground(opts)
        finally:
            sys.executable = old_exe
            sys.argv = old_argv
            sys.platform = old_platform

        self.expect(
            'processes.Process',
                (['foo/python2.3', 'python.exe', 'a', 'b', '--background',
                'c', '-b', 'bg-process'],),
                {'detach': True},
            'os._exit', 0
        )

    def test18handleBackground(self):
        """ ._handleBackground catches NotImplementedError on Win32 """
        self.mock.error = "processes.ni"
        opts = base.Store(background = True)
        self.addMock('signal', None)
        import sys
        old_platform = sys.platform
        old_argv = sys.argv
        try:
            sys.platform = "win32"
            sys.argv = ['python.exe', 'a', 'b', '--background', 'c', '-b']
            self._myhelper()._handleBackground(opts)
        finally:
            sys.argv = old_argv
            sys.platform = old_platform

        self.expect(
            'warnings.warn',
                'svnmailer: --background is not implemented on this platform',
                None, None
        )

    def test19delocalize(self):
        """ ._delocalize transforms path options to unicode """
        class MyHelper(self._myhelper):
            _PATH_OPTIONS = (('foo', '--foo'), ('bar', '--baz'))
        opts = base.Store(foo = '\xa4uro', bar = 'Andr\xe9',
            path_encoding = 'iso-8859-15')
        MyHelper()._delocalize(opts)
        self.mock.result.extend([opts.foo, opts.bar, opts.path_encoding])

        self.expect(
            'fromLocale', '\xa4uro', 'iso-8859-15',
            'fromLocale', 'Andr\xe9', 'iso-8859-15',
            'unicode(\xa4uro, iso-8859-15)', 'unicode(Andr\xe9, iso-8859-15)',
                'iso-8859-15'
        )

    def test20delocalize(self):
        """ ._delocalize converts UnicodeError to CommandlineError """
        self.mock.error = 'util.filename'
        class MyHelper(self._myhelper):
            _PATH_OPTIONS = (('foo', '--foo'),)
        opts = base.Store(foo = '\xa4uro', path_encoding = 'iso-8859-15')

        self.failUnlessRaises(self._cli.CommandlineError,
            MyHelper()._delocalize, opts)

    def test21ensureRequired(self):
        """ ._ensureRequired rejects missing options in certain modes """
        class MyHelper(self._myhelper):
            _REQUIRED_OPTIONS = (('foo', ('a', 'b'), 'text'),)
        opts = base.Store(foo = None, mode = 'a')

        self.failUnlessRaises(self._cli.CommandlineError,
            MyHelper()._ensureRequired, opts)

    def test22ensureRequired(self):
        """ ._ensureRequired rejects missing options in all modes """
        class MyHelper(self._myhelper):
            _REQUIRED_OPTIONS = (('foo', None, 'text'),)
        opts = base.Store(foo = None, mode = 'a')

        self.failUnlessRaises(self._cli.CommandlineError,
            MyHelper()._ensureRequired, opts)

    def test23ensureRequired(self):
        """ ._ensureRequired passes missing options in certain modes """
        class MyHelper(self._myhelper):
            _REQUIRED_OPTIONS = (('foo', ('b', 'c'), 'text'),)
        opts = base.Store(foo = None, mode = 'a')
        MyHelper()._ensureRequired(opts)
        
        self.expect()

    def test24addOptions(self):
        """ ._addOptions adds the option groups properly """
        class MyHelper(self._myhelper):
            def _addCommonOptions(this):
                self.mock.result.append("common")
            def _addBehaviorOptions(this):
                self.mock.result.append("behavior")
            def _addSupplementalOptions(this):
                self.mock.result.append("supplemental")
        MyHelper()._addOptions()

        self.expect("common", "behavior", "supplemental")

    def test25addCommonOptions(self):
        """ ._addCommonOptions adds the common options """
        self._myhelper()._addCommonOptions()

        self.expect(
            'Parser.add_option_group', ('title:common',), {},
            'OptionGroup.add_option', ('--debug',),
                {'action': 'store_true', 'default': False,
                'help': 'Run in debug mode (means basically that all '
                'messages are sent to STDOUT)'},
            'OptionGroup.add_option', ('-d', '--repository'),
                {'help': 'The repository directory'},
            'OptionGroup.add_option', ('-f', '--config'),
                {'help': 'The configuration file'},
            'OptionGroup.add_option', ('-e', '--path-encoding'),
                {'help': 'Specifies the character encoding to be used for '
                'filenames. By default the encoding is tried to be '
                'determined automatically depending on the locale.'},
            'OptionGroup.add_option', ('-b', '--background'),
                {'action': 'store_true', 'default': False,
                'help': 'Lets the mailer do its work in the background. '
                'That way the hook script can exit faster.'}
        )

    def test26addBehaviorOptions(self):
        """ ._addBehaviorOptions adds the behavior parameters for 1.2 """
        self._myhelper()._addBehaviorOptions()

        self.expect(
            'Parser.add_option_group', ('title:behavior',),
                {'description': 'The behavior options are mutually '
                'exclusive, i.e. the last one wins.'},
            'OptionGroup.add_option', ('-c', '--commit'),
                {'dest': 'mode', 'action': 'store_const', 'default': 'C',
                'const': 'C', 'help': 'This is a regular commit of '
                'versioned data (post-commit hook). This is default.'},
            'OptionGroup.add_option', ('-p', '--propchange'),
                {'dest': 'mode', 'action': 'store_const', 'const': 'P',
                'help': 'This is a modification of unversioned properties '
                '(post-revprop-change hook)'},
            'OptionGroup.add_option', ('-l', '--lock'),
                {'dest': 'mode', 'action': 'store_const', 'const': 'L',
                'help': '(svn 1.2 and later) This is a locking call '
                '(post-lock hook)'},
            'OptionGroup.add_option', ('-u', '--unlock'),
                {'dest': 'mode', 'action': 'store_const', 'const': 'U',
                'help': '(svn 1.2 and later) This is a unlocking call '
                '(post-unlock hook)'}
        )

    def test27addBehaviorOptions(self):
        """ ._addBehaviorOptions adds the behavior parameters for 1.1 """
        self.mock['svnmailer.subversion'].version.min_1_2 = False
        self._myhelper()._addBehaviorOptions()

        self.expect(
            'Parser.add_option_group', ('title:behavior',),
                {'description': 'The behavior options are mutually '
                'exclusive, i.e. the last one wins.'},
            'OptionGroup.add_option', ('-c', '--commit'),
                {'dest': 'mode', 'action': 'store_const', 'default': 'C',
                'const': 'C', 'help': 'This is a regular commit of '
                'versioned data (post-commit hook). This is default.'},
            'OptionGroup.add_option', ('-p', '--propchange'),
                {'dest': 'mode', 'action': 'store_const', 'const': 'P',
                'help': 'This is a modification of unversioned properties '
                '(post-revprop-change hook)'}
        )

    def test28addSupplementalOptions(self):
        """ ._addSupplementalOptions adds the supplemental options for 1.2 """
        self._myhelper()._addSupplementalOptions()

        self.expect(
            'Parser.add_option_group', ('title:supp',), {},
            'OptionGroup.add_option', ('-r', '--revision'),
                {'action': 'store', 'type': 'int',
                'help': 'The modified/committed revision number'},
            'OptionGroup.add_option', ('-a', '--author'),
                {'help': 'The author of the modification'},
            'OptionGroup.add_option', ('-n', '--propname'),
                {'help': 'The name of the modified property'},
            'OptionGroup.add_option', ('-o', '--action'),
                {'help': '(svn 1.2 and later) The property change action'}
        )

    def test29addSupplementalOptions(self):
        """ ._addSupplementalOptions adds the supplemental options for 1.1 """
        self.mock['svnmailer.subversion'].version.min_1_2 = False
        self._myhelper()._addSupplementalOptions()

        self.expect(
            'Parser.add_option_group', ('title:supp',), {},
            'OptionGroup.add_option', ('-r', '--revision'),
                {'action': 'store', 'type': 'int',
                'help': 'The modified/committed revision number'},
            'OptionGroup.add_option', ('-a', '--author'),
                {'help': 'The author of the modification'},
            'OptionGroup.add_option', ('-n', '--propname'),
                {'help': 'The name of the modified property'}
        )

    def test30transformArgs(self):
        """ ._transformArgs rejects empty command lines """
        self.failUnlessRaises(self._cli.CommandlineError,
            self._myhelper()._transformArgs, [])

    def test31transformArgs(self):
        """ ._transformArgs passes args on unknown modes with 1.2 """
        class MyHelper(self._myhelper):
            _OLD_OPTIONS = {}
            _OLD_OPTIONS_1_2 = {}
        self.mock.result.extend(MyHelper()._transformArgs(['foo', 'bar']))

        self.expect('foo', 'bar')

    def test32transformArgs(self):
        """ ._transformArgs passes args on unknown modes with 1.1 """
        self.mock['svnmailer.subversion'].version.min_1_2 = False
        class MyHelper(self._myhelper):
            _OLD_OPTIONS = {}
            _OLD_OPTIONS_1_2 = {'foo': ('--bar', '--config')}
        self.mock.result.extend(MyHelper()._transformArgs(['foo', 'bar']))

        self.expect('foo', 'bar')

    def test33transformArgs(self):
        """ ._transformArgs passes args on mismatching length """
        class MyHelper(self._myhelper):
            _OLD_OPTIONS = {'foo':
                ('--foo', '--bar', '--repository', '--config')}
            _OLD_OPTIONS_1_2 = {}
        self.mock.result.extend(MyHelper()._transformArgs(['foo', 'bar']))

        self.expect('foo', 'bar')

    def test34transformArgs(self):
        """ ._transformArgs transforms args with maximum length """
        class MyHelper(self._myhelper):
            _OLD_OPTIONS = {'foo':
                ('--foo', '--repository', '--config')}
            _OLD_OPTIONS_1_2 = {}
        self.mock.result.extend(
            MyHelper()._transformArgs(['foo', 'bar', 'baz'])
        )

        self.expect('--foo', '--repository', 'bar', '--config', 'baz')

    def test35transformArgs(self):
        """ ._transformArgs transforms args with minimum length """
        class MyHelper(self._myhelper):
            _OLD_OPTIONS = {'foo':
                ('--foo', '--repository', '--config')}
            _OLD_OPTIONS_1_2 = {}
        self.mock.result.extend(
            MyHelper()._transformArgs(['foo', 'bar'])
        )

        self.expect('--foo', '--repository', 'bar')

_tests.append(OptionHelperTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
